#ifndef __LIFEGAME_HPP
#define __LIFEGAME_HPP

#include "BPPLib.hpp"
#include "LifeGameApp.hpp"

class LifeGameData;

class Matrix {
private:
    int width;
    int height;
    BPPHeap heap;
public:
    Matrix() {};
    Matrix(int width_, int height_){
        init(width_, height);
    }
    int getWidth() { return width; }
    int getHeight() { return height; }
    void init(int width_, int height_){
        width = width_;
        height = height_;
        heap.Resize(width * height);
        for (int i = 0; i < width * height ; i++) {
            ((byte*)heap.GetHeap())[i] = 0;
        }
    }
    byte getCell(int x, int y) {
        if (0 <= x && x < width &&  0 <= y && y < height) {
            return ((byte*)heap.GetHeap())[x + (y) * width];
        } else {
            DBGPRINTF("error getCell");
            return 0;
        }
    }
    void setCell(int x, int y, byte life) {
        if (0 <= x && x < width &&  0 <= y && y < height) {
            ((byte*)heap.GetHeap())[x + (y) * width] = life;
        } else {
            DBGPRINTF("error setCell");
        }
    }
    void reverseCell(int x, int y) {
        if (0 <= x && x < width &&  0 <= y && y < height) {
            DBGPRINTF("%d", ((byte*)heap.GetHeap())[x + (y) * width]);
            ((byte*)heap.GetHeap())[x + (y) * width] = !((byte*)heap.GetHeap())[x + (y) * width];
            DBGPRINTF("%d", ((byte*)heap.GetHeap())[x + (y) * width]);
        } else {
            DBGPRINTF("error reverseCell");
        }
    }
    void operator=(Matrix& m) {
        width = m.width;
        height = m.height;
        heap.Resize(width * height);
        MEMCPY(heap, m.heap, width * height);
    }
};


class LifeGame {
public:
    static const int CELL_WIDTH = 4;
    static const int CELL_HEIGHT = 4;
private:
    static const int MATRIX_WIDTH = 50;
    static const int MATRIX_HEIGHT = 50;

    int width;
    int height;
    Matrix matrix[2];
    int current;

public:
    LifeGame();
    Matrix& getMatrix();
    int getWidth() { return width; }
    int getHeight() { return height; }
    void setSize(int width_, int height_);
    void next();
    void setMatrix(Matrix& copy);
    void paint(IDisplay* display);
};

#endif //__LIFEGAME_HPP
