#include "LifeGameApp.hpp"
#include "SoftKeyWindow.hpp"
#include "LifeGameMainPanel.hpp"
#include "LifeGame.hpp"
#include "LifeGameMainCanvas.hpp"
#include "LifeGameEditCanvas.hpp"
#include "SavePanel.hpp"
#include "LoadPanel.hpp"
#include "ChangeSizePanel.hpp"

static  boolean LifeGameApp_HandleEvent(LifeGameApp* pMe, 
                                                   AEEEvent eCode, uint16 wParam, 
                                                   uint32 dwParam);
boolean LifeGameApp_InitAppData(LifeGameApp* pMe);
void    LifeGameApp_FreeAppData(LifeGameApp* pMe);

int AEEClsCreateInstance(AEECLSID ClsId, IShell *pIShell, IModule *po, void **ppObj)
{
	*ppObj = NULL;
    if( ClsId == AEECLSID_LIFEGAMEAPP ) {
		if( AEEApplet_New(sizeof(LifeGameApp),ClsId,pIShell,po, (IApplet**)ppObj,
        (AEEHANDLER)LifeGameApp_HandleEvent, NULL) ) 	{
			if(LifeGameApp_InitAppData((LifeGameApp*)*ppObj)) {
				return(AEE_SUCCESS);
			} else {
				IAPPLET_Release((IApplet*)*ppObj);
				return EFAILED;
			}
        }
    }
	return(EFAILED);
}

static boolean LifeGameApp_HandleEvent(LifeGameApp* app, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
    switch (eCode) 
	{
    case EVT_APP_START:                        
        app->WindowManager = new WindowManager(app);
        app->WindowManager->setCurrent(new LifeGameMainPanel(app->WindowManager));
        return(TRUE);
    case EVT_APP_STOP:
        delete app->WindowManager;
        return(TRUE);
    case EVT_APP_SUSPEND: // {̓TXyhAW[sKv邪AB
        return(TRUE);
    case EVT_APP_RESUME:
        return(TRUE);
    case EVT_APP_MESSAGE:
        return(TRUE);
    case EVT_KEY:
        app->WindowManager->keyPress(wParam);
      	return(TRUE);
    default:
            break;
    }

    return FALSE;
}

boolean LifeGameApp_InitAppData(LifeGameApp* pMe)
{
    pMe->DeviceInfo.wStructSize = sizeof(pMe->DeviceInfo);
    ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->DeviceInfo);

    pMe->pIDisplay = pMe->a.m_pIDisplay;
    pMe->pIShell   = pMe->a.m_pIShell;

    return TRUE;
}

WindowManager::WindowManager(LifeGameApp* app) : display(app->pIDisplay), shell(app->pIShell)
{ 
    lifeGame = new LifeGame;

    AEEDeviceInfo devinfo;
    // IShellC^[tFCX֐gpāAʃTCY擾B
    ISHELL_GetDeviceInfo(shell, &devinfo);
    skwindow = new SoftKeyWindow(this, devinfo.cxScreen);
}

WindowManager::~WindowManager() {
    delete lifeGame;
    delete skwindow;
    delete frontWindow;
}

void WindowManager::changeState(STATE_ENUM state){
    switch(state){
    case STATE_GAMESTART:
        setCurrent(new LifeGameMainCanvas(this));
        break;
    case STATE_TITLE:
        setCurrent(new LifeGameMainPanel(this));
        break;
    case STATE_EDIT:
        setCurrent(new LifeGameEditCanvas(this));
        break;
    case STATE_SAVE:
        setCurrent(new SavePanel(this));
        break;
    case STATE_LOAD:
        setCurrent(new LoadPanel(this));
        break;
    case STATE_CHANGESIZE:
        setCurrent(new ChangeSizePanel(this));
        break;
    }
}

void WindowManager::setCurrent(Window* window) {
    delete frontWindow;
    frontWindow = window;
}

void WindowManager::update(IDisplay* disp) {
    skwindow->paint();
    IDISPLAY_Update(disp);
}
